
Object subclass: #BlockCipher
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cryptography-Core'!

!BlockCipher commentStamp: 'hmm 3/26/2002 18:17' prior: 0!
Abstract class for blockmode ciphers (those which encrypt/decrypt more than 1 byte at a time, typically 64 bits)!

BlockCipher subclass: #BlockCipherMode
	instanceVariableNames: 'cipher vector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cryptography-Core'!

!BlockCipherMode class methodsFor: 'accessing' stamp: 'len 8/9/2002 13:12'!
blockSize
	^ self shouldNotImplement! !

!BlockCipherMode class methodsFor: 'accessing' stamp: 'len 8/9/2002 13:12'!
keySize
	^ self shouldNotImplement! !

!BlockCipherMode class methodsFor: 'instance creation' stamp: 'len 8/3/2002 01:48'!
on: aCipher
	^ self new setCipher: aCipher! !

!BlockCipherMode methodsFor: 'accessing' stamp: 'len 8/3/2002 01:39'!
blockSize
	^ cipher blockSize! !

!BlockCipherMode methodsFor: 'accessing' stamp: 'len 8/7/2002 16:28'!
initialVector: aByteArray
	vector _ aByteArray! !

!BlockCipherMode methodsFor: 'accessing' stamp: 'len 8/3/2002 01:39'!
keySize
	^ cipher keySize! !

!BlockCipherMode methodsFor: 'accessing' stamp: 'len 8/3/2002 01:39'!
key: aByteArray
	cipher key: aByteArray! !

!BlockCipherMode methodsFor: 'private' stamp: 'cmm 3/4/2006 18:19'!
nonceBits
	^ self blockSize * 8! !

!BlockCipherMode methodsFor: 'initialization' stamp: 'cmm 3/4/2006 18:21'!
nonce: anInteger
	vector
		uint: self nonceBits
		at: 1
		put: anInteger! !

!BlockCipherMode methodsFor: 'printing' stamp: 'rww 11/19/2002 02:17'!
printOn: aStream
	aStream print: cipher; nextPut: $-; nextPutAll: self class name! !

!BlockCipherMode methodsFor: 'initialization' stamp: 'jcg 11/24/2006 13:59'!
setCipher: aCipher
	cipher := aCipher.
	vector := ByteArray new: self blockSize.
	cipher isInsecure ifTrue: [
		"Avoid accidental use of insecure cipher."
		InsecureCipherException signal 
	].
! !

!BlockCipher class methodsFor: 'accessing' stamp: 'len 8/9/2002 13:06'!
blockSize
	^ self subclassResponsibility! !

!BlockCipher class methodsFor: 'accessing' stamp: 'len 8/9/2002 13:06'!
keySize
	^ self subclassResponsibility! !

!BlockCipher class methodsFor: 'instance creation' stamp: 'len 8/9/2002 13:15'!
key: aByteArray
	^ self new key: aByteArray! !

!BlockCipher methodsFor: 'accessing' stamp: 'len 8/9/2002 13:05'!
blockSize
	^ self class blockSize! !

!BlockCipher methodsFor: 'private' stamp: 'jcg 11/24/2006 14:07'!
blocksIn: aByteArray from: startIndex to: endIndex do: oneArgBlock 
	| tempBlock myBlockSize |
	self isStreamCipher
		ifFalse: [ self validateEndpointsFrom: startIndex to: endIndex ].
	tempBlock := ByteArray new: self blockSize.
	myBlockSize := self blockSize.
	startIndex to: endIndex
		by: myBlockSize
		do: 
			[ : startPos | 
			tempBlock 
				replaceFrom: 1
				to: (myBlockSize min: aByteArray size - startPos + 1)
				with: aByteArray
				startingAt: startPos.
			oneArgBlock value: tempBlock.
			aByteArray 
				replaceFrom: startPos
				to: (startPos + myBlockSize - 1 min: aByteArray size)
				with: tempBlock
				startingAt: 1]! !

!BlockCipher methodsFor: 'modes' stamp: 'len 8/3/2002 01:47'!
cbc
	^ CBC on: self! !

!BlockCipher methodsFor: 'modes' stamp: 'len 8/3/2002 01:47'!
cfb
	^ CFB on: self! !

!BlockCipher methodsFor: 'modes' stamp: 'cmm 2/9/2006 13:14'!
ctr
	^ CTR on: self! !

!BlockCipher methodsFor: 'accessing' stamp: 'len 8/2/2002 03:13'!
decryptBlock: cipherText
	^ self subclassResponsibility! !

!BlockCipher methodsFor: 'accessing' stamp: 'cmm 3/4/2006 17:48'!
decrypt: aByteArray
	"Answer a copy of aByteArray which is decrypted with my key."
	| decryptedBlock |
	decryptedBlock _ 
		self 
			decrypt: aByteArray copy
			from: 1
			to: aByteArray size.
	^ self isStreamCipher 
		ifTrue: [ decryptedBlock ]
		ifFalse: 
			[ decryptedBlock
				copyFrom: 1 
				to: decryptedBlock unpaddedSize ]! !

!BlockCipher methodsFor: 'accessing' stamp: 'cmm 3/2/2006 23:53'!
decrypt: aByteArray from: startPos to: endPos 
	self 
		blocksIn: aByteArray
		from: startPos
		to: endPos
		do: [:each | self decryptBlock: each].
	^ aByteArray! !

!BlockCipher methodsFor: 'modes' stamp: 'len 8/3/2002 01:51'!
ecb
	^ self! !

!BlockCipher methodsFor: 'accessing' stamp: 'len 8/2/2002 03:13'!
encryptBlock: plainText
	^ self subclassResponsibility! !

!BlockCipher methodsFor: 'accessing' stamp: 'cmm 3/3/2006 00:05'!
encrypt: aByteArray 
	"Answer a copy of aByteArray which is encrypted with my key."
	| encryptable |
	encryptable _ self isStreamCipher
		ifTrue: [ aByteArray copy ]
		ifFalse: [ aByteArray padToMultipleOf: self blockSize ].
	^ self 
		encrypt: encryptable
		from: 1
		to: encryptable size! !

!BlockCipher methodsFor: 'accessing' stamp: 'cmm 3/2/2006 23:39'!
encrypt: aByteArray from: startPos to: endPos 
	self 
		blocksIn: aByteArray
		from: startPos
		to: endPos
		do: [:each | self encryptBlock: each].
	^aByteArray! !

!BlockCipher methodsFor: 'testing' stamp: 'jcg 11/24/2006 13:58'!
isInsecure
	"Hopefully not!!"
	^ false! !

!BlockCipher methodsFor: 'testing' stamp: 'cmm 3/2/2006 23:50'!
isStreamCipher
	"Answer whether I have stream characteristics; a Stream cipher typically generates its own stream of pseudo-random bytes which is then XOR'd with plaintext.  Another characteristic is no padding is required because the last (odd-sized) block can simply XOR to whatever length needed."
	^ false! !

!BlockCipher methodsFor: 'accessing' stamp: 'len 8/9/2002 13:05'!
keySize
	^ self class keySize! !

!BlockCipher methodsFor: 'accessing' stamp: 'len 8/2/2002 02:41'!
key: aKey
	^ self subclassResponsibility! !

!BlockCipher methodsFor: 'modes' stamp: 'len 8/3/2002 01:48'!
ofb
	^ OFB on: self! !

!BlockCipher methodsFor: 'private' stamp: 'cmm 3/2/2006 23:22'!
validateEndpointsFrom: startIndex to: endIndex
	endIndex-startIndex+1 \\ self blockSize = 0 ifFalse: [ CryptographyError signal: 'message size must be a multiple of my blockSize.' ]! !

!ByteArray class methodsFor: '*cryptography-core' stamp: 'RJT 8/6/2006 22:05'!
fromHexString: hexString 
	"Returns a byteArray from a hex string. The hex string can have
	leading zeros that will not be truncated, preserving the size of the
	byteArray for things like cyrptograpy"
	| anInteger hexSize stringSize aStringOfHex |
	aStringOfHex := (hexString copyWithoutAll: (Array with: $  with: Character cr with: Character tab)).
	stringSize _ aStringOfHex size.
	(stringSize > 3
			and: [(aStringOfHex copyFrom: 1 to: 3) = '16r'])
		ifTrue: [anInteger _ Number readFrom: (aStringOfHex asUppercase at: 3 put: $r;
							 yourself).
			hexSize _ aStringOfHex size - 3 / 2]
		ifFalse: [anInteger _ Number readFrom: aStringOfHex asUppercase base: 16.
			hexSize _ aStringOfHex size / 2].
	^ anInteger asByteArrayOfSize: hexSize asInteger! !

!ByteArray methodsFor: '*cryptography-core' stamp: 'rww 4/11/2004 14:48'!
asByteArrayOfSize: size
	"
		'34523' asByteArray asByteArrayOfSize: 100.

	(((
		| repeats bytes | 
		repeats := 1000000.
		bytes := '123456789123456789123456789123456789123456789123456789' asByteArray.
		 [repeats timesRepeat: (bytes asByteArrayOfSize: 1024) ] timeToRun.
	)))"

	| bytes |
	size < self size
		ifTrue: [^ self error: 'bytearray bigger than ', size asString].
	bytes := self asByteArray.
	^ (ByteArray new: (size - bytes size)), bytes
! !

!ByteArray methodsFor: '*cryptography-core' stamp: 'len 10/15/2002 18:45'!
asInteger
	^ self inject: 0 into: [ :sum :each | sum * 256 + each]! !

!ByteArray methodsFor: '*cryptography-core' stamp: 'cmm 2/16/2006 23:15'!
bitXor: aByteArray
	| answer |
	answer _ self copy.
	1 to: (self size min: aByteArray size) do: 
		[ :each | 
		answer 
			at: each 
			put: ((self at: each) bitXor: (aByteArray at: each)) ].
	^ answer! !

!ByteArray methodsFor: '*cryptography-core' stamp: 'cmm 2/21/2006 00:05'!
destroy
	1 to: self size do: 
		[ : x |
		self at: x put: 0 ]! !

!ByteArray methodsFor: '*cryptography-core' stamp: 'len 6/16/2001 22:25'!
hex
	| stream |
	stream _ '' writeStream.
	self do: [ :each |
		stream
			nextPut: ('0123456789ABCDEF' at: each // 16 + 1);
			nextPut: ('0123456789ABCDEF' at: each \\ 16 + 1)].
	^ stream contents! !

!ByteArray methodsFor: '*cryptography-core' stamp: 'rww 1/12/2003 22:08'!
hexStringInNibbles


	^ (self  
		inject: String new writeStream
		into: [:stream :byte |
			stream
				nextPut: ('0123456789ABCDEF' at: byte // 16 + 1);
				nextPut: ('0123456789ABCDEF' at: byte \\ 16 + 1);
				nextPut: Character space.  
			stream]) contents.
! !

!ByteArray methodsFor: '*cryptography-core' stamp: 'cmm 3/2/2006 21:14'!
isProperlyPadded
	"Verify my bytes have padding applied by #padToMultipleOf:.  The padding scheme used here is option #2 as specified by Ferguson and Schneier in 'Practical Cryptography', p. 68-69."
	| pad end start |
	pad _ self last.
	end _ self size.
	start _ end - pad + 1.
	^ start > 0
	and: [ (start to: end) allSatisfy: [ : each | (self at: each) = pad ] ]! !

!ByteArray methodsFor: '*cryptography-core' stamp: 'cmm 3/2/2006 12:57'!
paddedSize: blockSize
	"If my size is an exact multiple of blockSize, an entire pad-block is required.  That's why we have the +1."
	^ (self size+1) roundUpTo: blockSize! !

!ByteArray methodsFor: '*cryptography-core' stamp: 'cmm 3/2/2006 21:28'!
padToMultipleOf: blockSize
	"Answer a new ByteArray that is padded to anInteger bytes.  This is used by block-based ciphers where the length of the ByteArray to be encrypted must be a multiple of that ciphers #blockSize.  The padding scheme used here is option #2 as specified by Ferguson and Schneier in 'Practical Cryptography', p. 68-69."
	| answer pad |
	answer _ ByteArray new: (self paddedSize: blockSize).
	answer
		replaceFrom: 1
		to: self size
		with: self
		startingAt: 1.
	pad _ answer size - self size.
	self size+1 
		to: answer size 
		do: [ : x | answer at: x put: pad ].
	^ answer! !

!ByteArray methodsFor: '*cryptography-core' stamp: 'rww 1/13/2003 02:12'!
toEBase64String

	| base64Table stream firstIndex secondIndex thirdIndex fourthIndex extra |
	base64Table := '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz=_'.

	stream := String new writeStream.
	1 to: self size - 2 by: 3 do: [:i |
		firstIndex := ((self at: i) bitAnd: 16rFC) >> 2.
		secondIndex := ((((self at: i) bitAnd: 16r03) << 4) bitOr: (((self at: i+1) bitAnd: 16rF0) >> 4)).
		thirdIndex := ((((self at: i+1) bitAnd: 16r0F) << 2) bitOr: (((self at: i+2) bitAnd: 16rC0) >> 6)).
		fourthIndex := ((self at: i+2) bitAnd: 16r3F).
		stream nextPut: (base64Table at: firstIndex + 1).
		stream nextPut: (base64Table at: secondIndex + 1).
		stream nextPut: (base64Table at: thirdIndex + 1).
		stream nextPut: (base64Table at: fourthIndex + 1)].

	extra := self size - (self size // 3 * 3).
	extra = 0 ifTrue: [^ stream contents].
	extra = 1 ifTrue: [
		firstIndex := ((self at: self size) bitAnd: 16rFC) >> 2.
		secondIndex := ((self at: self size) bitAnd: 16r03) << 4.
		stream nextPut: (base64Table at: firstIndex + 1).
		stream nextPut: (base64Table at: secondIndex + 1).
		^ stream contents].
	extra = 2 ifTrue: [
		firstIndex := ((self at: self size - 1) bitAnd: 16rFC) >> 2.
		secondIndex := ((((self at: self size - 1) bitAnd: 16r03) << 4) bitOr: (((self at: self size) bitAnd: 16rF0) >> 4)).
		thirdIndex := ((self at: self size) bitAnd: 16r0F) << 2.
		stream nextPut: (base64Table at: firstIndex + 1).
		stream nextPut: (base64Table at: secondIndex + 1).
		stream nextPut: (base64Table at: thirdIndex + 1).
		^ stream contents].
! !

!ByteArray methodsFor: '*cryptography-core' stamp: 'cmm 2/23/2006 22:59'!
uint: bits at: anInteger

	| answer bytes |
	bits == 64 ifTrue: [ ^ self unsigned64At: anInteger ].
	bits == 56 ifTrue: [ ^ self unsigned56At: anInteger ].
	bits == 48 ifTrue: [ ^ self unsigned48At: anInteger ].
	bits == 40 ifTrue: [ ^ self unsigned40At: anInteger ].
	bits == 32
		ifTrue:
			[ ^ self
				unsignedLongAt: anInteger
				bigEndian: false ].
	bits == 16
		ifTrue:
			[ ^ self
				unsignedShortAt: anInteger
				bigEndian: false ].
	bits == 8 
		ifTrue:
			[ ^ self byteAt: anInteger ].
	bytes _ bits // 8.
	answer _ LargePositiveInteger new: bytes.
	1 to: bytes do:
		[ :digitPosition |
		answer
			digitAt: digitPosition
			put: (self at: digitPosition + anInteger - 1) ].
	^answer normalize! !

!ByteArray methodsFor: '*cryptography-core' stamp: 'cmm 2/23/2006 13:32'!
uint: bits at: position put: anInteger

	position
		to: position + (bits // 8) - 1
		do:
			[ :pos |
			self
				at: pos
				put: (anInteger digitAt: pos-position+1) ].
	^ anInteger! !

!ByteArray methodsFor: '*cryptography-core' stamp: 'cmm 3/3/2006 00:08'!
unpad
	| pad end |
	self isProperlyPadded ifFalse: [ CryptographyError signal: 'Authentication failure (improperly padded!!)' ].
	pad _ self last.
	end _ self size - pad.
	^ self copyFrom: 1 to: end! !

!ByteArray methodsFor: '*cryptography-core' stamp: 'cmm 3/4/2006 15:08'!
unpaddedSize
	"If I was padded by a BlockCipher, answer the size of the original plaintext."
	self isProperlyPadded ifFalse: [ CryptographyError signal: 'Authentication failure (improperly padded!!)' ].
	^ self size - self last! !

!ByteArray methodsFor: '*cryptography-core' stamp: 'cmm 2/23/2006 13:25'!
unsigned40At: index
	"Return a unsigned integer quantity starting from the given byte index"

	| b0 b1 b2 b3 w |

	w _ self at: index.
	b3 _ self at: index+1.
	b2 _ self at: index+2.
	b1 _ self at: index+3.
	b0 _ self at: index+4.

	"Minimize LargeInteger arithmetic"
	b3 = 0 ifFalse:[w _ (b3 bitShift: 8) + w].
	b2 = 0 ifFalse:[w _ (b2 bitShift: 16) + w].
	b1 = 0 ifFalse:[w _ (b1 bitShift: 24) + w].
	b0 = 0 ifFalse:[w _ (b0 bitShift: 32) + w].

	^w! !

!ByteArray methodsFor: '*cryptography-core' stamp: 'cmm 2/23/2006 13:25'!
unsigned48At: index
	"Return a unsigned integer quantity starting from the given byte index"

	| b0 b1 b2 b3 b4 w |

	w _ self at: index.
	b4 _ self at: index+1.
	b3 _ self at: index+2.
	b2 _ self at: index+3.
	b1 _ self at: index+4.
	b0 _ self at: index+5.

	"Minimize LargeInteger arithmetic"
	b4 = 0 ifFalse:[w _ (b4 bitShift: 8) + w].
	b3 = 0 ifFalse:[w _ (b3 bitShift: 16) + w].
	b2 = 0 ifFalse:[w _ (b2 bitShift: 24) + w].
	b1 = 0 ifFalse:[w _ (b1 bitShift: 32) + w].
	b0 = 0 ifFalse:[w _ (b0 bitShift: 40) + w].

	^w! !

!ByteArray methodsFor: '*cryptography-core' stamp: 'cmm 2/23/2006 13:25'!
unsigned56At: index
	"Return a unsigned integer quantity starting from the given byte index"

	| b0 b1 b2 b3 b4 b5 w |

	w _ self at: index.
	b5 _ self at: index+1.
	b4 _ self at: index+2.
	b3 _ self at: index+3.
	b2 _ self at: index+4.
	b1 _ self at: index+5.
	b0 _ self at: index+6.

	"Minimize LargeInteger arithmetic"
	b5 = 0 ifFalse:[w _ (b5 bitShift: 8) + w].
	b4 = 0 ifFalse:[w _ (b4 bitShift: 16) + w].
	b3 = 0 ifFalse:[w _ (b3 bitShift: 24) + w].
	b2 = 0 ifFalse:[w _ (b2 bitShift: 32) + w].
	b1 = 0 ifFalse:[w _ (b1 bitShift: 40) + w].
	b0 = 0 ifFalse:[w _ (b0 bitShift: 48) + w].

	^w! !

!ByteArray methodsFor: '*cryptography-core' stamp: 'cmm 2/23/2006 13:25'!
unsigned64At: index
	"Return a unsigned integer quantity starting from the given byte index"

	| b0 b1 b2 b3 b4 b5 b6 w |

	w _ self at: index.
	b6 _ self at: index+1.
	b5 _ self at: index+2.
	b4 _ self at: index+3.
	b3 _ self at: index+4.
	b2 _ self at: index+5.
	b1 _ self at: index+6.
	b0 _ self at: index+7.

	"Minimize LargeInteger arithmetic"
	b6 = 0 ifFalse:[w _ (b6 bitShift: 8) + w].
	b5 = 0 ifFalse:[w _ (b5 bitShift: 16) + w].
	b4 = 0 ifFalse:[w _ (b4 bitShift: 24) + w].
	b3 = 0 ifFalse:[w _ (b3 bitShift: 32) + w].
	b2 = 0 ifFalse:[w _ (b2 bitShift: 40) + w].
	b1 = 0 ifFalse:[w _ (b1 bitShift: 48) + w].
	b0 = 0 ifFalse:[w _ (b0 bitShift: 56) + w].

	^w! !

BlockCipherMode subclass: #CBC
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cryptography-Core'!

!CBC methodsFor: 'accessing' stamp: 'jcg 11/24/2006 02:26'!
decryptBlock: aByteArray
	| nextVector |
	nextVector := aByteArray clone.
	cipher decryptBlock: aByteArray.
	1 to: aByteArray size do: [ :i | aByteArray at: i put: ((vector at: i) bitXor: (aByteArray at: i))].
	vector := nextVector! !

!CBC methodsFor: 'accessing' stamp: 'jcg 11/24/2006 02:25'!
encryptBlock: aByteArray
	1 to: aByteArray size do: [ :i | aByteArray at: i put: ((vector at: i) bitXor: (aByteArray at: i))].
	cipher encryptBlock: aByteArray.
	vector := aByteArray clone! !

!CBC methodsFor: 'initialization' stamp: 'cmm 3/4/2006 18:43'!
nonce: anInteger
	super nonce: anInteger.
	"Ferguson and Schneier recommend this on page 72 of 'Practical Cryptography'."
	cipher encryptBlock: vector! !

BlockCipherMode subclass: #CFB
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cryptography-Core'!

!CFB methodsFor: 'accessing' stamp: 'jcg 11/24/2006 02:26'!
decryptBlock: aByteArray
	| nextVector |
	nextVector := aByteArray clone.
	cipher encryptBlock: vector.
	1 to: aByteArray size do: [ :i | aByteArray at: i put: ((vector at: i) bitXor: (aByteArray at: i))].
	vector := nextVector! !

!CFB methodsFor: 'accessing' stamp: 'jcg 11/24/2006 02:25'!
encryptBlock: aByteArray
	cipher encryptBlock: vector.
	1 
		to: aByteArray size 
		do: 
			[ :i | 
			aByteArray 
				at: i 
				put: ((vector at: i) bitXor: (aByteArray at: i)) ].
	vector := aByteArray clone! !

!CFB methodsFor: 'testing' stamp: 'cmm 3/7/2006 21:32'!
isStreamCipher
	"Because the actual ciphertext plays a role in computing the next vector, streaming mode seems inappropriate for CFB.  There's also an implementation issue; the vector is shortened after the last (irregularly sized) block, rendering that BlockCipher instance unusable.."
	^ false! !

BlockCipherMode subclass: #CTR
	instanceVariableNames: 'counter counterBits maximumCounterValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cryptography-Core'!

!CTR commentStamp: '<historical>' prior: 0!
This CTR mode implementation was guided by

	Nils Ferguson, Bruce Schneier.  Pratical Cryptography.  
	Wiley, 2003.
	pp. 75-82, 111-127.

With CTR, my initialVector is partitioned into a nonce and a counter ("i" in the book).  The book suggests the nonce would be the message-number used also for sequencing messages.  The overall requirement is that the same initialVector (i.e., counter+nonce combination) never be used twice for this key (instance).  The counter is re-set to 1 each time the nonce is set.

My blockSize is 128 bits.  If you run out of counter, I signal a CryptographyUsageError.!

!CTR methodsFor: 'accessing' stamp: 'cmm 3/1/2006 22:46'!
asMakoEncryptable: aByteArray
	^ aByteArray copy! !

!CTR methodsFor: 'accessing' stamp: 'cmm 2/20/2006 23:07'!
counter
	^ counter! !

!CTR methodsFor: 'initialization' stamp: 'cmm 2/26/2006 22:16'!
counterBits: anInteger
	((16 to: 128 by: 16) includes: anInteger) ifFalse:
		[ CryptographyError signal: 'invalid counter size' ].
	counterBits _ anInteger.
	maximumCounterValue _ (2 raisedTo: counterBits) - 1 ! !

!CTR methodsFor: 'private' stamp: 'cmm 2/13/2006 20:02'!
counterPosition
	"For a MAC, p. 122 of Practical Cryptography suggests a 4-byte counter number, followed by a 4-byte message-number and then eight zero bytes.  I think the eight zero bytes are for 'future flexibility', but not sure, and we need to handle key renegotiation anyway.  This means we can encipher up to (2 raisedTo: 32) blocks or an equivalent number of messages."
	^1! !

!CTR methodsFor: 'private' stamp: 'cmm 2/26/2006 21:15'!
counter: anInteger
	vector
		uint: counterBits
		at: self counterPosition
		put: anInteger.
	counter _ anInteger! !

!CTR methodsFor: 'accessing' stamp: 'cmm 2/8/2006 23:57'!
decryptBlock: aByteArray
	"One of the advantages of CTR mode is that the decrypt implementation is exactly like the encrypt."
	self encryptBlock: aByteArray! !

!CTR methodsFor: 'accessing' stamp: 'cmm 2/20/2006 23:45'!
encryptBlock: aByteArray
	| kSubI |
	kSubI _ self nextBlock.
	aByteArray withIndexDo:
		[ : eachByte : x |
		aByteArray 
			at: x
			put: (eachByte bitXor: (kSubI at: x)).
		"Be extra paranoid and overwrite the key block."
		kSubI at: x put: 0 ]! !

!CTR methodsFor: 'private' stamp: 'cmm 2/28/2006 13:06'!
incrementCounter
	counter >= maximumCounterValue 
		ifTrue:
			[ CryptographyError signal: 'cannot operate on more than ', maximumCounterValue printString, ' blocks at a time.' ].
	self counter: counter + 1! !

!CTR methodsFor: 'initialization' stamp: 'cmm 2/26/2006 22:11'!
initialize
	super initialize.
	self initializeForFortuna! !

!CTR methodsFor: 'initialization' stamp: 'cmm 2/26/2006 22:13'!
initializeForFortuna
	"Configure me for use in the Fortuna PRNG as specified by Ferguson and Schneier in 'Practical Cryptography', chapter 10."
	self counterBits: 128! !

!CTR methodsFor: 'initialization' stamp: 'cmm 2/26/2006 22:12'!
initializeForSecureChannel
	"Configure me for use in a secure-channel as suggested by Ferguson and Schneier in 'Practical Cryptography', chapter 8."
	self counterBits: 32! !

!CTR methodsFor: 'testing' stamp: 'cmm 3/2/2006 23:51'!
isStreamCipher
	^ true! !

!CTR methodsFor: 'private' stamp: 'cmm 2/23/2006 13:55'!
messageNumberPosition
	"4 byte block-number first (the counter), then the 4-byte message-number."
	^ 5! !

!CTR methodsFor: 'secure channel' stamp: 'cmm 2/28/2006 21:44'!
messageNumber: anInteger
	"For a MAC, p. 122 of Practical Cryptography suggests a 4-byte counter number, followed by a 4-byte message-number and then eight zero bytes.  I think the eight zero bytes are for 'future flexibility', but not sure, and we need to handle key renegotiation anyway.  This means we can encipher up to (2 raisedTo: 32) blocks or an equivalent number of messages before having to get a new key."
	vector
		longAt: self messageNumberPosition
		put: anInteger
		bigEndian: false.
	self counter: 0! !

!CTR methodsFor: 'Fortuna' stamp: 'cmm 2/20/2006 23:45'!
nextBlock
	| kSubI |
	self incrementCounter.
	kSubI _ vector copy.  "as named in the book"
	cipher encryptBlock: kSubI.
	^ kSubI! !

!CTR methodsFor: 'private' stamp: 'cmm 3/4/2006 18:20'!
nonceBits
	^ counterBits! !

!CTR methodsFor: 'private' stamp: 'cmm 3/4/2006 18:20'!
nonce: anInteger
	super nonce: anInteger.
	counter _ anInteger! !

!CTR methodsFor: 'initialization' stamp: 'cmm 2/26/2006 22:11'!
setCipher: aCipher
	super setCipher: aCipher.
	self counter: 0! !

Object subclass: #HashFunction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cryptography-Core'!

!HashFunction class methodsFor: 'accessing' stamp: 'len 8/15/2002 01:43'!
blockSize
	^ self subclassResponsibility! !

!HashFunction class methodsFor: 'hashing' stamp: 'len 8/2/2002 02:20'!
hashMessage: aStringOrByteArray
	^ self new hashMessage: aStringOrByteArray! !

!HashFunction class methodsFor: 'accessing' stamp: 'len 8/9/2002 13:17'!
hashSize
	^ self subclassResponsibility! !

!HashFunction class methodsFor: 'hashing' stamp: 'len 8/2/2002 02:20'!
hashStream: aPositionableStream
	^ self new hashStream: aPositionableStream! !

!HashFunction methodsFor: 'accessing' stamp: 'len 8/15/2002 01:43'!
blockSize
	^ self class blockSize! !

!HashFunction methodsFor: 'accessing' stamp: 'cmm 2/20/2006 23:22'!
doubleHashMessage: aStringOrByteArray
	"SHA1 new doubleHashMessage: 'foo'"
	^ self doubleHashStream: aStringOrByteArray asByteArray readStream! !

!HashFunction methodsFor: 'accessing' stamp: 'cmm 2/20/2006 23:21'!
doubleHashStream: aStream
	^ self hashStream: ((self hashStream: aStream) asByteArray readStream)! !

!HashFunction methodsFor: 'accessing' stamp: 'len 8/7/2002 16:30'!
hashMessage: aStringOrByteArray
	"MD5 new hashMessage: 'foo'"
	^ self hashStream: aStringOrByteArray asByteArray readStream! !

!HashFunction methodsFor: 'accessing' stamp: 'len 8/9/2002 13:17'!
hashSize
	^ self class hashSize! !

!HashFunction methodsFor: 'accessing' stamp: 'len 8/2/2002 02:21'!
hashStream: aStream
	^ self subclassResponsibility! !

!HashFunction methodsFor: 'converting' stamp: 'len 8/3/2002 02:42'!
hmac
	^ HMAC on: self! !

Object subclass: #HMAC
	instanceVariableNames: 'hash key ipad epad'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cryptography-Core'!

!HMAC commentStamp: '<historical>' prior: 0!
HMAC is a mechanism for message authentication using cryptographic hash functions. HMAC can be used with any iterative cryptographic hash function, e.g., MD5, SHA-1, in combination with a secret shared key.  The cryptographic strength of HMAC depends on the properties of the underlying hash function.

See RFC 2114.!

!HMAC class methodsFor: 'instance creation' stamp: 'len 8/15/2002 01:42'!
on: aHashFunction
	^ self new setHash: aHashFunction! !

!HMAC methodsFor: 'accessing' stamp: 'len 10/16/2002 16:43'!
digestMessage: aByteArray
	^ hash hashMessage: (key bitXor: epad), (hash hashMessage: (key bitXor: ipad), aByteArray)! !

!HMAC methodsFor: 'accessing' stamp: 'len 8/3/2002 02:06'!
digestSize
	^ hash hashSize! !

!HMAC methodsFor: 'accessing' stamp: 'len 10/16/2002 16:47'!
key: aByteArray
	key _ aByteArray.
	key size > hash blockSize ifTrue: [key _ hash hashMessage: key].
	key size < hash blockSize ifTrue: [key _ key, (ByteArray new: hash blockSize - key size)]! !

!HMAC methodsFor: 'printing' stamp: 'len 8/3/2002 02:08'!
printOn: aStream
	aStream nextPutAll: 'HMAC-'; print: hash! !

!HMAC methodsFor: 'initialization' stamp: 'len 10/16/2002 16:42'!
setHash: aHash
	hash _ aHash.
	ipad _ ByteArray new: aHash blockSize withAll: 16r36.
	epad _ ByteArray new: aHash blockSize withAll: 16r5C! !

BlockCipher subclass: #InsecureBlockCipher
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cryptography-Core'!

!InsecureBlockCipher commentStamp: 'jcg 11/24/2006 14:02' prior: 0!
This is an abstact superclass for block ciphers that are, for some reason, known to be insecure.  Two possible reasons that an insecure cipher might exist are:
	- a new attack is discovered against a cipher that was formerly thought to be secure
	- a trivial cipher might be used for testing purposes.!

!InsecureBlockCipher methodsFor: 'testing' stamp: 'jcg 11/24/2006 13:58'!
isInsecure
	^ true! !

Exception subclass: #InsecureCipherException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cryptography-Core'!

!Integer methodsFor: '*cryptography-core' stamp: 'rww 12/7/2002 23:27'!
areAllBitsSet

	^self digitLength ~= (self + 1) digitLength! !

!Integer methodsFor: '*cryptography-core' stamp: 'rww 1/2/2003 00:30'!
asArray

	| stream |
	stream := WriteStream on: Array new.
	self digitLength to: 1 by: -1 do: [:digitIndex |
		stream nextPut: (self digitAt: digitIndex)].
	^ stream contents
! !

!Integer methodsFor: '*cryptography-core' stamp: 'rww 1/2/2003 00:29'!
asByteArray

	| stream |
	stream := WriteStream on: ByteArray new.
	self digitLength to: 1 by: -1 do: [:digitIndex |
		stream nextPut: (self digitAt: digitIndex)].
	^ stream contents
! !

!Integer methodsFor: '*cryptography-core' stamp: 'rww 9/21/2006 12:43'!
asByteArrayOfSize: aSize

	| stream |
	aSize < self digitLength ifTrue: [self error: 'number to large for byte array'].
	stream := WriteStream on: ByteArray new.
	aSize - self digitLength timesRepeat: [stream nextPut: 0].
	self digitLength to: 1 by: -1 do: [:digitIndex |
		stream nextPut: (self digitAt: digitIndex)].
	^ stream contents
! !

!Integer methodsFor: '*cryptography-core' stamp: 'len 11/14/2002 11:51'!
inverseModulo: n
	"Answer the inverse of the receiver modulus n. That is, the integer y such that (self * y) \\ n is 1. Both self and n must be positive, and it is assumed that self < n and that n is integer."
	"Details: Use the extended Euclidean algorithm, Schneier, p. 247."

	| v u k u1 u2 u3 t1 t2 t3 tmp |
	((self <= 0) or: [n <= 0]) ifTrue: [self error: 'number and modulo must be greater than zero'].
	self >= n ifTrue: [self error: 'number must be < modulo'].

	v _ self.
	u _ n.
	k _ 0.
	[self even and: [n even and: [u > 0]]] whileTrue: [  "eliminate common factors of two"
		k _ k + 1.
		u _ u bitShift: -1.
		v _ v bitShift: -1].

	u1 _ 1. u2 _ 0. u3 _ u.
	t1 _ v. t2 _ u - 1. t3 _ v.
	[	[u3 even ifTrue: [
			((u1 odd) or: [u2 odd]) ifTrue: [
				u1 _ u1 + v.
				u2 _ u2 + u].
			u1 _ u1 bitShift: -1.
			u2 _ u2 bitShift: -1.
			u3 _ u3 bitShift: -1].
		((t3 even) or: [u3 < t3]) ifTrue: [
			tmp _ u1. u1 _ t1. t1 _ tmp.
			tmp _ u2. u2 _ t2. t2 _ tmp.
			tmp _ u3. u3 _ t3. t3 _ tmp].
		u3 even and: [u3 > 0]] whileTrue: ["loop while u3 is even"].

		[((u1 < t1) or: [u2 < t2]) and: [u1 > 0]] whileTrue: [
			u1 _ u1 + v.
			u2 _ u2 + u].
	
		u1 _ u1 - t1.
		u2 _ u2 - t2.
		u3 _ u3 - t3.
		t3 > 0] whileTrue: ["loop while t3 > 0"].

	[u1 >= v and: [u2 >= u]] whileTrue: [
		u1 _ u1 - v.
		u2 _ u2 - u].

	u1 _ u1 bitShift: k.
	u2 _ u2 bitShift: k.
	u3 _ u3 bitShift: k.

	u3 = 1 ifFalse: [self error: 'no inverse'].
	^ u - u2! !

!Integer methodsFor: '*cryptography-core' stamp: 'cs 12/5/2002 15:47'!
raisedTo: y modulo: n
	"Answer the modular exponential. Code by Jesse Welton."
	| s t u |
	s _ 1.
	t _ self.
	u _ y.
	[u = 0] whileFalse: [
		u odd ifTrue: [
			s _ s * t.
			s >= n ifTrue: [s _ s \\\ n]].
		t _ t * t.
		t >= n ifTrue: [t _ t \\\ n].
		u _ u bitShift: -1].
	^ s normalize
! !

!Integer methodsFor: '*cryptography-core' stamp: 'rww 1/13/2003 01:22'!
toEBase64String

	self < 0 ifTrue: [^ '-', self negated toEBase64String].
	^ self asByteArray toEBase64String.
! !

!Integer methodsFor: '*cryptography-core' stamp: 'rww 12/7/2002 23:28'!
twosComplement
	| complement digit length |
	length := self digitLength + 1.
	complement := Integer new: length neg: false.
	1 to: length-1 do: [:index |
		digit := self digitAt: index.
		digit := digit bitXor: 16rFF.	"flip bits without 2's complement logic in bitInvert"
		complement digitAt: index put: digit].	
	self areAllBitsSet ifTrue: [complement digitAt: length put: 16rFF].
	^complement + 1! !

!LargePositiveInteger methodsFor: '*cryptography-core' stamp: 'cmm 2/14/2006 23:20'!
destroy
	"Wipe my bytes so that they cannot possibly be discovered."
	1 to: self size do:
		[ : x |
		self 
			digitAt: x
			put: 0 ]! !

BlockCipherMode subclass: #OFB
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cryptography-Core'!

!OFB methodsFor: 'private' stamp: 'cmm 3/1/2006 23:04'!
advanceVector
	cipher encryptBlock: vector! !

!OFB methodsFor: 'accessing' stamp: 'cmm 3/1/2006 22:45'!
asMakoEncryptable: aByteArray
	^ aByteArray copy! !

!OFB methodsFor: 'accessing' stamp: 'cmm 3/1/2006 22:44'!
decryptBlock: aByteArray
	"One of the nice attributes of OFB is there is no difference between the encrypt and decrypt implementations."
	^ self encryptBlock: aByteArray! !

!OFB methodsFor: 'accessing' stamp: 'cmm 3/1/2006 23:02'!
encryptBlock: aByteArray
	self advanceVector.
	aByteArray withIndexDo:
		[ : eachByte : x |
		aByteArray 
			at: x
			put: ((aByteArray at: x) bitXor: (vector at: x)) ]! !

!OFB methodsFor: 'testing' stamp: 'cmm 3/2/2006 23:51'!
isStreamCipher
	^ true! !

!PositionableStream methodsFor: '*cryptography-core' stamp: 'rww 1/13/2003 00:10'!
nextUtfString

	^ (self next: self uint16) asString.
! !

!PositionableStream methodsFor: '*cryptography-core' stamp: 'rww 1/13/2003 00:10'!
nextUtfStringPut: s 

	self uint16: s size.
	self nextPutAll: s asByteArray.
! !

!SocketStream methodsFor: '*Cryptography-Core' stamp: 'RJT 7/15/2006 21:19'!
peekInBufferForAll: aString
	"Answer whether or not the inbuffer in the receiver
	contains aString. If a match is made answer true. 
	We use findString:startingAt: to avoid copying."
	self receiveAvailableData.
	^(inBuffer findString: aString) > 0
		! !

!ThirtyTwoBitRegister class methodsFor: '*cryptography-core-instance creation' stamp: ''!
byte1: hi1 byte2: hi2 byte3: low1 byte4: low2
	^ super new byte1: hi1 byte2: hi2 byte3: low1 byte4: low2! !

!ThirtyTwoBitRegister class methodsFor: '*cryptography-core-instance creation' stamp: ''!
loadFrom: aByteArray at: index
	^ super new loadFrom: aByteArray at: index! !

!ThirtyTwoBitRegister class methodsFor: '*cryptography-core-instance creation' stamp: 'jm 12/14/1999 16:05'!
new
	"Answer a new instance whose initial contents is zero."

	^ super new load: 0
! !

!ThirtyTwoBitRegister methodsFor: '*cryptography-core-accessing' stamp: 'len 8/7/2002 17:37'!
asByteArray
	^ ByteArray with: (low bitAnd: 16rFF) with: (low bitShift: -8) with: (hi bitAnd: 16rFF) with: (hi bitShift: -8)! !

!ThirtyTwoBitRegister methodsFor: '*cryptography-core-accessing' stamp: 'jm 12/14/1999 16:03'!
asInteger
	"Answer the integer value of my current contents."

	^ (hi bitShift: 16) + low
! !

!ThirtyTwoBitRegister methodsFor: '*cryptography-core-accessing' stamp: 'DSM 1/20/2000 17:17'!
asReverseInteger
	"Answer the byte-swapped integer value of my current contents."

	^ ((low bitAnd: 16rFF) bitShift: 24) +
       ((low bitAnd: 16rFF00) bitShift: 8) +
	  ((hi bitAnd: 16rFF) bitShift: 8) +
       (hi bitShift: -8)
! !

!ThirtyTwoBitRegister methodsFor: '*cryptography-core-accumulator ops' stamp: 'jm 12/7/1999 15:41'!
bitAnd: aThirtTwoBitRegister
	"Replace my contents with the bitwise AND of the given register and my current contents."

	hi _ hi bitAnd: aThirtTwoBitRegister hi.
	low _ low bitAnd: aThirtTwoBitRegister low.
! !

!ThirtyTwoBitRegister methodsFor: '*cryptography-core-accumulator ops' stamp: 'jm 12/7/1999 15:40'!
bitInvert
	"Replace my contents with the bitwise inverse my current contents."

	hi _ hi bitXor: 16rFFFF.
	low _ low bitXor: 16rFFFF.
! !

!ThirtyTwoBitRegister methodsFor: '*cryptography-core-accumulator ops' stamp: 'jm 12/7/1999 15:40'!
bitOr: aThirtTwoBitRegister
	"Replace my contents with the bitwise OR of the given register and my current contents."

	hi _ hi bitOr: aThirtTwoBitRegister hi.
	low _ low bitOr: aThirtTwoBitRegister low.
! !

!ThirtyTwoBitRegister methodsFor: '*cryptography-core-accumulator ops' stamp: 'RJT 10/28/2005 15:42'!
bitShift: anInteger
	"Replace my contents with the bitShift of anInteger."
	self load: (self asInteger bitShift: anInteger). ! !

!ThirtyTwoBitRegister methodsFor: '*cryptography-core-accumulator ops' stamp: 'jm 12/7/1999 15:38'!
bitXor: aThirtTwoBitRegister
	"Replace my contents with the bitwise exclusive OR of the given register and my current contents."

	hi _ hi bitXor: aThirtTwoBitRegister hi.
	low _ low bitXor: aThirtTwoBitRegister low.
! !

!ThirtyTwoBitRegister methodsFor: '*cryptography-core-accessing' stamp: 'cs 8/7/2002 16:12'!
byte1: hi1 byte2: hi2 byte3: low1 byte4: low2
	hi _ (hi1 bitShift: 8) + hi2.
	low _ (low1 bitShift: 8) + low2.! !

!ThirtyTwoBitRegister methodsFor: '*cryptography-core-accessing' stamp: 'len 8/15/2002 01:34'!
byteAt: anInteger
	anInteger = 1 ifTrue: [^ hi bitShift: -8].
	anInteger = 2 ifTrue: [^ hi bitAnd: 16rFF].
	anInteger = 3 ifTrue: [^ low bitShift: -8].
	anInteger = 4 ifTrue: [^ low bitAnd: 16rFF]! !

!ThirtyTwoBitRegister methodsFor: '*cryptography-core-copying' stamp: 'jm 12/7/1999 15:26'!
copy
	"Use the clone primitive for speed."

	<primitive: 148>
	^ super copy
! !

!ThirtyTwoBitRegister methodsFor: '*cryptography-core-accessing' stamp: 'jm 12/7/1999 15:26'!
hi

	^ hi
! !

!ThirtyTwoBitRegister methodsFor: '*cryptography-core-accumulator ops' stamp: 'jm 12/7/1999 23:09'!
leftRotateBy: bits
	"Rotate my contents left by the given number of bits, retaining exactly 32 bits."
	"Details: Perform this operation with as little LargeInteger arithmetic as possible."

	| bitCount s1 s2 newHi |
	"ensure bitCount is in range [0..32]"
	bitCount _ bits \\ 32.
	bitCount < 0 ifTrue: [bitCount _ bitCount + 32].

	bitCount > 16
		ifTrue: [
			s1 _ bitCount - 16.
			s2 _ s1 - 16.
			newHi _ ((low bitShift: s1) bitAnd: 16rFFFF) bitOr: (hi bitShift: s2).
			low _ ((hi bitShift: s1) bitAnd: 16rFFFF) bitOr: (low bitShift: s2).
			hi _ newHi]
		ifFalse: [
			s1 _ bitCount.
			s2 _ s1 - 16.
			newHi _ ((hi bitShift: s1) bitAnd: 16rFFFF) bitOr: (low bitShift: s2).
			low _ ((low bitShift: s1) bitAnd: 16rFFFF) bitOr: (hi bitShift: s2).
			hi _ newHi]
! !

!ThirtyTwoBitRegister methodsFor: '*cryptography-core-accessing' stamp: 'jm 12/14/1999 16:07'!
loadFrom: aByteArray at: index
	"Load my 32-bit value from the four bytes of the given ByteArray starting at the given index. Consider the first byte to contain the most significant bits of the word (i.e., use big-endian byte ordering)."

	hi _ ((aByteArray at: index) bitShift: 8) + ( aByteArray at: index + 1).
	low _ ((aByteArray at: index + 2) bitShift: 8) + ( aByteArray at: index + 3).
! !

!ThirtyTwoBitRegister methodsFor: '*cryptography-core-accessing' stamp: 'jm 12/14/1999 16:07'!
load: anInteger
	"Set my contents to the value of given integer."

	low _ anInteger bitAnd: 16rFFFF.
	hi _ (anInteger bitShift: -16) bitAnd: 16rFFFF.
	self asInteger = anInteger
		ifFalse: [self error: 'out of range: ', anInteger printString].
! !

!ThirtyTwoBitRegister methodsFor: '*cryptography-core-accessing' stamp: 'jm 12/7/1999 15:26'!
low

	^ low! !

!ThirtyTwoBitRegister methodsFor: '*cryptography-core-printing' stamp: 'jm 12/14/1999 16:05'!
printOn: aStream
	"Print my contents in hex with a leading 'R' to show that it is a register object being printed."

	aStream nextPutAll: 'R:'.
	self asInteger printOn: aStream base: 16.
! !

!ThirtyTwoBitRegister methodsFor: '*cryptography-core-accessing' stamp: 'mdr 2/23/2001 22:51'!
reverseLoadFrom: aByteArray at: index
	"Load my 32-bit value from the four bytes of the given ByteArray
starting at the given index. Consider the first byte to contain the most
significant bits of the word (i.e., use big-endian byte ordering)."

	hi _ ((aByteArray at: index + 3) bitShift: 8) + ( aByteArray at: index + 2).
	low _ ((aByteArray at: index + 1) bitShift: 8) + ( aByteArray at: index).
! !

!ThirtyTwoBitRegister methodsFor: '*cryptography-core-accessing' stamp: 'len 8/15/2002 01:29'!
storeInto: aByteArray at: index
	"Store my 32-bit value into the four bytes of the given ByteArray starting at the given index. Consider the first byte to contain the most significant bits of the word (i.e., use big-endian byte ordering)."

	aByteArray at: index put: (hi bitShift: -8).
	aByteArray at: index + 1 put: (hi bitAnd: 16rFF).
	aByteArray at: index + 2 put: (low bitShift: -8).
	aByteArray at: index + 3 put: (low bitAnd: 16rFF)! !

!ThirtyTwoBitRegister methodsFor: '*cryptography-core-accumulator ops' stamp: 'jm 12/7/1999 15:36'!
+= aThirtTwoBitRegister
	"Replace my contents with the sum of the given register and my current contents."

	| lowSum |
	lowSum _ low + aThirtTwoBitRegister low.
	hi _ (hi + aThirtTwoBitRegister hi + (lowSum bitShift: -16)) bitAnd: 16rFFFF.
	low _ lowSum bitAnd: 16rFFFF.
! !
SystemOrganization addCategory: #'Cryptography-Core'!
